/* File: RoombaKarel.java
 * 
 * A Karel program in which Karel picks up all the beepers in a
 * square world.  Karel is assumed to start at (1, 1) and each
 * corner will have no more than one beeper on it.
 */

import stanford.karel.*;

public class RoombaKarel extends SuperKarel {
	public void run() {
		sweepOneRow();
		while (leftIsClear()) {
			moveToNextRow();
			sweepOneRow();
		}
	}
	
	/* Makes Karel sweep up the current row and come back home.
	 * 
	 * Precondition:  Karel is at the start of the row, facing East.
	 * Postcondition: Karel is at the start of the row, facing East,
	 *                and Karel has cleaned all the beepers in this row.
	 */
	private void sweepOneRow() {
		sweepToEndOfRow();
		comeHome();
	}
	
	/* Makes Karel sweep up the current row and end at the other side.
	 * 
	 * Precondition:  Karel is at the start of the row, facing East.
	 * Postcondition: Karel is at the end of the row, facing East,
	 *                and all the beepers in this row are clear.
	 */
	private void sweepToEndOfRow() {
		while (frontIsClear()) {
			while (beepersPresent()) {
				pickBeeper();
			}
			move();
		}
		while (beepersPresent()) {
			pickBeeper();
		}
	}
	
	/* Makes Karel move back to the start of the row.
	 * 
	 * Precondition:  Karel is at the end of the row, facing East.
	 * Postcondition: Karel is at the start of the row, facing East.
	 */
	private void comeHome() {
		turnAround();
		while (frontIsClear()) {
			move();
		}
		turnAround();
	}
	
	/* Makes Karel move from the current row to the next row.
	 *
	 * Precondition:  Karel is facing East at the start of a row
	 *                that isn't the top row.
	 * Postcondition: Karel is facting East at the start of the
	 *                row above the previous row.
	 */
	private void moveToNextRow() {
		turnLeft();
		move();
		turnRight();
	}
	
}
